# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: openfeature/schemas/protobuf/sync/v1/sync_service.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'openfeature/schemas/protobuf/sync/v1/sync_service.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n7openfeature/schemas/protobuf/sync/v1/sync_service.proto\x12\x07sync.v1\"9\n\x10SyncFlagsRequest\x12\x13\n\x0bprovider_id\x18\x01 \x01(\t\x12\x10\n\x08selector\x18\x02 \x01(\t\"R\n\x11SyncFlagsResponse\x12\x1a\n\x12\x66lag_configuration\x18\x01 \x01(\t\x12!\n\x05state\x18\x02 \x01(\x0e\x32\x12.sync.v1.SyncState\"=\n\x14\x46\x65tchAllFlagsRequest\x12\x13\n\x0bprovider_id\x18\x01 \x01(\t\x12\x10\n\x08selector\x18\x02 \x01(\t\"3\n\x15\x46\x65tchAllFlagsResponse\x12\x1a\n\x12\x66lag_configuration\x18\x01 \x01(\t*\x92\x01\n\tSyncState\x12\x1a\n\x16SYNC_STATE_UNSPECIFIED\x10\x00\x12\x12\n\x0eSYNC_STATE_ALL\x10\x01\x12\x12\n\x0eSYNC_STATE_ADD\x10\x02\x12\x15\n\x11SYNC_STATE_UPDATE\x10\x03\x12\x15\n\x11SYNC_STATE_DELETE\x10\x04\x12\x13\n\x0fSYNC_STATE_PING\x10\x05\x32\xab\x01\n\x0f\x46lagSyncService\x12\x46\n\tSyncFlags\x12\x19.sync.v1.SyncFlagsRequest\x1a\x1a.sync.v1.SyncFlagsResponse\"\x00\x30\x01\x12P\n\rFetchAllFlags\x12\x1d.sync.v1.FetchAllFlagsRequest\x1a\x1e.sync.v1.FetchAllFlagsResponse\"\x00\x42\x97\x01\n\x1a\x64\x65v.openfeature.flagd.syncZ\x0e\x66lagd/grpcsync\xb8\x01\x01\xaa\x02\x16OpenFeature.Flagd.Sync\xca\x02\'OpenFeature\\Providers\\Flagd\\Schema\\Sync\xea\x02\"OpenFeature::FlagD::Provider::Syncb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'openfeature.schemas.protobuf.sync.v1.sync_service_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\032dev.openfeature.flagd.syncZ\016flagd/grpcsync\270\001\001\252\002\026OpenFeature.Flagd.Sync\312\002\'OpenFeature\\Providers\\Flagd\\Schema\\Sync\352\002\"OpenFeature::FlagD::Provider::Sync'
  _globals['_SYNCSTATE']._serialized_start=328
  _globals['_SYNCSTATE']._serialized_end=474
  _globals['_SYNCFLAGSREQUEST']._serialized_start=68
  _globals['_SYNCFLAGSREQUEST']._serialized_end=125
  _globals['_SYNCFLAGSRESPONSE']._serialized_start=127
  _globals['_SYNCFLAGSRESPONSE']._serialized_end=209
  _globals['_FETCHALLFLAGSREQUEST']._serialized_start=211
  _globals['_FETCHALLFLAGSREQUEST']._serialized_end=272
  _globals['_FETCHALLFLAGSRESPONSE']._serialized_start=274
  _globals['_FETCHALLFLAGSRESPONSE']._serialized_end=325
  _globals['_FLAGSYNCSERVICE']._serialized_start=477
  _globals['_FLAGSYNCSERVICE']._serialized_end=648
# @@protoc_insertion_point(module_scope)
