# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: openfeature/schemas/protobuf/flagd/sync/v1/sync.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'openfeature/schemas/protobuf/flagd/sync/v1/sync.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5openfeature/schemas/protobuf/flagd/sync/v1/sync.proto\x12\rflagd.sync.v1\x1a\x1cgoogle/protobuf/struct.proto\"9\n\x10SyncFlagsRequest\x12\x13\n\x0bprovider_id\x18\x01 \x01(\t\x12\x10\n\x08selector\x18\x02 \x01(\t\"/\n\x11SyncFlagsResponse\x12\x1a\n\x12\x66lag_configuration\x18\x01 \x01(\t\"=\n\x14\x46\x65tchAllFlagsRequest\x12\x13\n\x0bprovider_id\x18\x01 \x01(\t\x12\x10\n\x08selector\x18\x02 \x01(\t\"3\n\x15\x46\x65tchAllFlagsResponse\x12\x1a\n\x12\x66lag_configuration\x18\x01 \x01(\t\"\x14\n\x12GetMetadataRequest\"F\n\x13GetMetadataResponse\x12)\n\x08metadata\x18\x02 \x01(\x0b\x32\x17.google.protobuf.StructJ\x04\x08\x01\x10\x02\x32\x9b\x02\n\x0f\x46lagSyncService\x12R\n\tSyncFlags\x12\x1f.flagd.sync.v1.SyncFlagsRequest\x1a .flagd.sync.v1.SyncFlagsResponse\"\x00\x30\x01\x12\\\n\rFetchAllFlags\x12#.flagd.sync.v1.FetchAllFlagsRequest\x1a$.flagd.sync.v1.FetchAllFlagsResponse\"\x00\x12V\n\x0bGetMetadata\x12!.flagd.sync.v1.GetMetadataRequest\x1a\".flagd.sync.v1.GetMetadataResponse\"\x00\x42\xa8\x01\n\x1f\x64\x65v.openfeature.flagd.grpc.syncZ\rflagd/sync/v1\xaa\x02\x1bOpenFeature.Flagd.Grpc.Sync\xca\x02,OpenFeature\\Providers\\Flagd\\Schema\\Grpc\\Sync\xea\x02(OpenFeature::Flagd::Provider::Grpc::Syncb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'openfeature.schemas.protobuf.flagd.sync.v1.sync_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\037dev.openfeature.flagd.grpc.syncZ\rflagd/sync/v1\252\002\033OpenFeature.Flagd.Grpc.Sync\312\002,OpenFeature\\Providers\\Flagd\\Schema\\Grpc\\Sync\352\002(OpenFeature::Flagd::Provider::Grpc::Sync'
  _globals['_SYNCFLAGSREQUEST']._serialized_start=102
  _globals['_SYNCFLAGSREQUEST']._serialized_end=159
  _globals['_SYNCFLAGSRESPONSE']._serialized_start=161
  _globals['_SYNCFLAGSRESPONSE']._serialized_end=208
  _globals['_FETCHALLFLAGSREQUEST']._serialized_start=210
  _globals['_FETCHALLFLAGSREQUEST']._serialized_end=271
  _globals['_FETCHALLFLAGSRESPONSE']._serialized_start=273
  _globals['_FETCHALLFLAGSRESPONSE']._serialized_end=324
  _globals['_GETMETADATAREQUEST']._serialized_start=326
  _globals['_GETMETADATAREQUEST']._serialized_end=346
  _globals['_GETMETADATARESPONSE']._serialized_start=348
  _globals['_GETMETADATARESPONSE']._serialized_end=418
  _globals['_FLAGSYNCSERVICE']._serialized_start=421
  _globals['_FLAGSYNCSERVICE']._serialized_end=704
# @@protoc_insertion_point(module_scope)
