# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: openfeature/schemas/protobuf/schema/v1/schema.proto
# Protobuf Python Version: 5.29.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    0,
    '',
    'openfeature/schemas/protobuf/schema/v1/schema.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n3openfeature/schemas/protobuf/schema/v1/schema.proto\x12\tschema.v1\x1a\x1cgoogle/protobuf/struct.proto\"=\n\x11ResolveAllRequest\x12(\n\x07\x63ontext\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct\"\x8f\x01\n\x12ResolveAllResponse\x12\x37\n\x05\x66lags\x18\x01 \x03(\x0b\x32(.schema.v1.ResolveAllResponse.FlagsEntry\x1a@\n\nFlagsEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12!\n\x05value\x18\x02 \x01(\x0b\x32\x12.schema.v1.AnyFlag:\x02\x38\x01\"\xaa\x01\n\x07\x41nyFlag\x12\x0e\n\x06reason\x18\x01 \x01(\t\x12\x0f\n\x07variant\x18\x02 \x01(\t\x12\x14\n\nbool_value\x18\x03 \x01(\x08H\x00\x12\x16\n\x0cstring_value\x18\x04 \x01(\tH\x00\x12\x16\n\x0c\x64ouble_value\x18\x05 \x01(\x01H\x00\x12/\n\x0cobject_value\x18\x06 \x01(\x0b\x32\x17.google.protobuf.StructH\x00\x42\x07\n\x05value\"S\n\x15ResolveBooleanRequest\x12\x10\n\x08\x66lag_key\x18\x01 \x01(\t\x12(\n\x07\x63ontext\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\"s\n\x16ResolveBooleanResponse\x12\r\n\x05value\x18\x01 \x01(\x08\x12\x0e\n\x06reason\x18\x02 \x01(\t\x12\x0f\n\x07variant\x18\x03 \x01(\t\x12)\n\x08metadata\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\"R\n\x14ResolveStringRequest\x12\x10\n\x08\x66lag_key\x18\x01 \x01(\t\x12(\n\x07\x63ontext\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\"r\n\x15ResolveStringResponse\x12\r\n\x05value\x18\x01 \x01(\t\x12\x0e\n\x06reason\x18\x02 \x01(\t\x12\x0f\n\x07variant\x18\x03 \x01(\t\x12)\n\x08metadata\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\"Q\n\x13ResolveFloatRequest\x12\x10\n\x08\x66lag_key\x18\x01 \x01(\t\x12(\n\x07\x63ontext\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\"q\n\x14ResolveFloatResponse\x12\r\n\x05value\x18\x01 \x01(\x01\x12\x0e\n\x06reason\x18\x02 \x01(\t\x12\x0f\n\x07variant\x18\x03 \x01(\t\x12)\n\x08metadata\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\"O\n\x11ResolveIntRequest\x12\x10\n\x08\x66lag_key\x18\x01 \x01(\t\x12(\n\x07\x63ontext\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\"o\n\x12ResolveIntResponse\x12\r\n\x05value\x18\x01 \x01(\x03\x12\x0e\n\x06reason\x18\x02 \x01(\t\x12\x0f\n\x07variant\x18\x03 \x01(\t\x12)\n\x08metadata\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\"R\n\x14ResolveObjectRequest\x12\x10\n\x08\x66lag_key\x18\x01 \x01(\t\x12(\n\x07\x63ontext\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\"\x8b\x01\n\x15ResolveObjectResponse\x12&\n\x05value\x18\x01 \x01(\x0b\x32\x17.google.protobuf.Struct\x12\x0e\n\x06reason\x18\x02 \x01(\t\x12\x0f\n\x07variant\x18\x03 \x01(\t\x12)\n\x08metadata\x18\x04 \x01(\x0b\x32\x17.google.protobuf.Struct\"J\n\x13\x45ventStreamResponse\x12\x0c\n\x04type\x18\x01 \x01(\t\x12%\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct\"\x14\n\x12\x45ventStreamRequest2\xcd\x04\n\x07Service\x12K\n\nResolveAll\x12\x1c.schema.v1.ResolveAllRequest\x1a\x1d.schema.v1.ResolveAllResponse\"\x00\x12W\n\x0eResolveBoolean\x12 .schema.v1.ResolveBooleanRequest\x1a!.schema.v1.ResolveBooleanResponse\"\x00\x12T\n\rResolveString\x12\x1f.schema.v1.ResolveStringRequest\x1a .schema.v1.ResolveStringResponse\"\x00\x12Q\n\x0cResolveFloat\x12\x1e.schema.v1.ResolveFloatRequest\x1a\x1f.schema.v1.ResolveFloatResponse\"\x00\x12K\n\nResolveInt\x12\x1c.schema.v1.ResolveIntRequest\x1a\x1d.schema.v1.ResolveIntResponse\"\x00\x12T\n\rResolveObject\x12\x1f.schema.v1.ResolveObjectRequest\x1a .schema.v1.ResolveObjectResponse\"\x00\x12P\n\x0b\x45ventStream\x12\x1d.schema.v1.EventStreamRequest\x1a\x1e.schema.v1.EventStreamResponse\"\x00\x30\x01\x42\x9a\x01\n\x1a\x64\x65v.openfeature.flagd.grpcZ\x11schema/service/v1\xb8\x01\x01\xaa\x02\x16OpenFeature.Flagd.Grpc\xca\x02\'OpenFeature\\Providers\\Flagd\\Schema\\Grpc\xea\x02\"OpenFeature::FlagD::Provider::Grpcb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'openfeature.schemas.protobuf.schema.v1.schema_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\032dev.openfeature.flagd.grpcZ\021schema/service/v1\270\001\001\252\002\026OpenFeature.Flagd.Grpc\312\002\'OpenFeature\\Providers\\Flagd\\Schema\\Grpc\352\002\"OpenFeature::FlagD::Provider::Grpc'
  _globals['_RESOLVEALLRESPONSE_FLAGSENTRY']._loaded_options = None
  _globals['_RESOLVEALLRESPONSE_FLAGSENTRY']._serialized_options = b'8\001'
  _globals['_RESOLVEALLREQUEST']._serialized_start=96
  _globals['_RESOLVEALLREQUEST']._serialized_end=157
  _globals['_RESOLVEALLRESPONSE']._serialized_start=160
  _globals['_RESOLVEALLRESPONSE']._serialized_end=303
  _globals['_RESOLVEALLRESPONSE_FLAGSENTRY']._serialized_start=239
  _globals['_RESOLVEALLRESPONSE_FLAGSENTRY']._serialized_end=303
  _globals['_ANYFLAG']._serialized_start=306
  _globals['_ANYFLAG']._serialized_end=476
  _globals['_RESOLVEBOOLEANREQUEST']._serialized_start=478
  _globals['_RESOLVEBOOLEANREQUEST']._serialized_end=561
  _globals['_RESOLVEBOOLEANRESPONSE']._serialized_start=563
  _globals['_RESOLVEBOOLEANRESPONSE']._serialized_end=678
  _globals['_RESOLVESTRINGREQUEST']._serialized_start=680
  _globals['_RESOLVESTRINGREQUEST']._serialized_end=762
  _globals['_RESOLVESTRINGRESPONSE']._serialized_start=764
  _globals['_RESOLVESTRINGRESPONSE']._serialized_end=878
  _globals['_RESOLVEFLOATREQUEST']._serialized_start=880
  _globals['_RESOLVEFLOATREQUEST']._serialized_end=961
  _globals['_RESOLVEFLOATRESPONSE']._serialized_start=963
  _globals['_RESOLVEFLOATRESPONSE']._serialized_end=1076
  _globals['_RESOLVEINTREQUEST']._serialized_start=1078
  _globals['_RESOLVEINTREQUEST']._serialized_end=1157
  _globals['_RESOLVEINTRESPONSE']._serialized_start=1159
  _globals['_RESOLVEINTRESPONSE']._serialized_end=1270
  _globals['_RESOLVEOBJECTREQUEST']._serialized_start=1272
  _globals['_RESOLVEOBJECTREQUEST']._serialized_end=1354
  _globals['_RESOLVEOBJECTRESPONSE']._serialized_start=1357
  _globals['_RESOLVEOBJECTRESPONSE']._serialized_end=1496
  _globals['_EVENTSTREAMRESPONSE']._serialized_start=1498
  _globals['_EVENTSTREAMRESPONSE']._serialized_end=1572
  _globals['_EVENTSTREAMREQUEST']._serialized_start=1574
  _globals['_EVENTSTREAMREQUEST']._serialized_end=1594
  _globals['_SERVICE']._serialized_start=1597
  _globals['_SERVICE']._serialized_end=2186
# @@protoc_insertion_point(module_scope)
